//
// turing - simulation of Turing machines
// Copyright (C) 2009  Joachim Reichel <joachim.reichel@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include "alphabet.h"

namespace TM {



Alphabet::Alphabet( const Character* alphabet)
{
    TM_ALWAYS_CHECK( alphabet);
    m_alphabet = alphabet;

    for( size_t i = 0; i < 256; ++i)
        m_character_index[i] = static_cast<size_t>( -1);

    for( size_t i = 0; i < m_alphabet.size(); ++i)
        m_character_index[ static_cast<size_t>( m_alphabet[i])] = i;

    TM_ALWAYS_CHECK( m_character_index[ static_cast<size_t>( Blank)]
        != static_cast<size_t>( -1));
}

size_t Alphabet::get_character_index( Character character) const
{
    size_t index = m_character_index[ static_cast<size_t>( character) ];
    TM_ALWAYS_CHECK( index != static_cast<size_t>( -1));
    return index;
}

bool Alphabet::operator==( const Alphabet& rhs) const
{
    return m_alphabet == rhs.m_alphabet;
}

bool Alphabet::operator!=( const Alphabet& rhs) const
{
    return !( *this == rhs);
}

std::ostream& operator<< (std::ostream& stream, const Alphabet& alphabet)
{
    stream << alphabet.m_alphabet;
    return stream;
}



} // namespace TM
