//
// turing - simulation of Turing machines
// Copyright (C) 2009  Joachim Reichel <joachim.reichel@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include "turing.h"

#include <fstream>
#include <string>
#include <iostream>
#include <sstream>
#include <algorithm>

void print_stats(
    const std::string& prefix,
    const TM::Turing_machine& tm,
    TM::Encoding encoding)
{
   std::cout << prefix
       << " initial state: " << tm.get_initial_state() << std::endl;
   std::cout << prefix
       << " halting state: " << tm.get_halting_state() << std::endl;
   std::cout << prefix
       << " alphabet:      " << tm.get_alphabet() << std::endl;
   std::cout << prefix
       << " #states:       " << tm.get_nr_of_states() << std::endl;
   std::cout << prefix
       << " #transitions:  " << tm.get_nr_of_transitions() << std::endl;
   std::cout << prefix
       << " encoding:      " << encoding << std::endl;
   std::cout << std::endl;
}

int main( int argc, char* argv[]) {

    TM_ALWAYS_CHECK( argc == 1+3);

    std::ifstream stream( argv[1]);
    TM::Encoding encoding( static_cast<TM::Encoding>( atoi( argv[2])));
    TM::Alphabet alphabet( argv[3]);
    TM::Turing_machine* tm = TM::ungoedel( stream, encoding, alphabet);
    TM_ALWAYS_CHECK( tm);

    print_stats( "X", *tm, encoding);

    delete tm;

    return 0;
}
